@@ -27,6 +27,7 @@ class Logger:
     def __init__(self, name):
         self.name = name
         self.handlers = None
+        self.parent = None
 
     def _level_str(self, level):
         l = _level_dict.get(level)
@@ -38,16 +39,19 @@ class Logger:
         self.level = level
 
     def isEnabledFor(self, level):
-        return level >= (self.level or _level)
+        return level >= self.level
 
     def log(self, level, msg, *args):
-        if level >= (self.level or _level):
+        dest = self
+        while dest.level == NOTSET and dest.parent:
+            dest = dest.parent
+        if level >= dest.level:
             record = LogRecord(
                 self.name, level, None, None, msg, args, None, None, None
             )
 
-            if self.handlers:
-                for hdlr in self.handlers:
+            if dest.handlers:
+                for hdlr in dest.handlers:
                     hdlr.emit(record)
 
     def debug(self, msg, *args):
@@ -81,22 +85,14 @@ class Logger:
         self.handlers.append(hdlr)
 
 
-_level = INFO
-_loggers = {}
-
-
 def getLogger(name=None):
     if name is None:
         name = "root"
     if name in _loggers:
         return _loggers[name]
-    if name == "root":
-        l = Logger(name)
-        sh = StreamHandler()
-        sh.formatter = Formatter()
-        l.addHandler(sh)
-    else:
-        l = Logger(name)
+    l = Logger(name)
+    # For now, we have shallow hierarchy, where parent of each logger is root.
+    l.parent = root
     _loggers[name] = l
     return l
 
@@ -121,8 +117,7 @@ def exception(msg, *args):
     getLogger(None).exception(msg, *args)
 
 def basicConfig(level=INFO, filename=None, stream=None, format=None, style="%"):
-    global _level
-    _level = level
+    root.setLevel(level)
     if filename:
         h = FileHandler(filename)
     else:
@@ -236,9 +231,6 @@ class Formatter:
         raise NotImplementedError()
 
 
-root = getLogger()
-
-
 class LogRecord:
     def __init__(
         self, name, level, pathname, lineno, msg, args, exc_info, func=None, sinfo=None
@@ -256,3 +248,11 @@ class LogRecord:
         self.exc_info = exc_info
         self.func = func
         self.sinfo = sinfo
+
+
+root = Logger("root")
+root.setLevel(WARNING)
+sh = StreamHandler()
+sh.formatter = Formatter()
+root.addHandler(sh)
+_loggers = {"root": root}
